@@ -23,7 +23,7 @@ logging.basicConfig(level=logging.INFO)
 Run, Instance, Detection, Product, Source = None, None, None, None, None
 SourceDetection, Comment, Tag, TagSourceDetection = None, None, None, None
 Observation, ObservationMetadata, Tile, Postprocessing = None, None, None, None
-KinematicModel = None
+KinematicModel, WKAPPProduct = None, None
 
 
 def connect(db="/mnt/shared/wallaby/apps/WALLABY_database", env="/mnt/shared/wallaby/apps/WALLABY_data_access/wallaby_data_access/.env"):
@@ -34,7 +34,7 @@ def connect(db="/mnt/shared/wallaby/apps/WALLABY_database", env="/mnt/shared/wal
     global Run, Instance, Detection, Product, Source
     global SourceDetection, Comment, Tag, TagSourceDetection
     global Observation, ObservationMetadata, Tile, Postprocessing
-    global KinematicModel
+    global KinematicModel, WKAPPProduct
     load_dotenv(env)
     sys.path.append(db)
     sys.path.append(db + "/orm")
@@ -42,7 +42,7 @@ def connect(db="/mnt/shared/wallaby/apps/WALLABY_database", env="/mnt/shared/wal
     from source_finding.models import Run, Instance, Detection, Product, Source
     from source_finding.models import SourceDetection, Comment, Tag, TagSourceDetection
     from operations.models import Observation, ObservationMetadata, Tile, Postprocessing
-    from kinematic_model.models import KinematicModel
+    from kinematic_model.models import KinematicModel, WKAPPProduct
     return
 
 
@@ -70,23 +70,40 @@ def _write_zipped_fits_file(filename, product, compress=True):
                 os.system(f'gzip {filename}')
 
 
-def _write_products(products, prefix):
-    _write_zipped_fits_file('%s_cube.fits' % (prefix), products.cube)
-    _write_zipped_fits_file('%s_chan.fits' % (prefix), products.chan)
-    _write_zipped_fits_file('%s_mask.fits' % (prefix), products.mask)
-    _write_zipped_fits_file('%s_mom0.fits' % (prefix), products.mom0)
-    _write_zipped_fits_file('%s_mom1.fits' % (prefix), products.mom1)
-    _write_zipped_fits_file('%s_mom2.fits' % (prefix), products.mom2)
+def _write_source_products(products, output_dir, name):
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_cube.fits'), products.cube)
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_chan.fits'), products.chan)
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_mask.fits'), products.mask)
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_mom0.fits'), products.mom0)
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_mom1.fits'), products.mom1)
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_mom2.fits'), products.mom2)
 
     # Open spectrum
     with io.BytesIO() as buf:
         buf.write(b''.join(products.spec))
         buf.seek(0)
-        spec_file = '%s_spec.txt' % (prefix)
+        spec_file = os.path.join(output_dir, f'{name}_spec.txt')
         if not os.path.isfile(spec_file):
             _write_bytesio_to_file(spec_file, buf)
 
 
+def _write_kinematic_model_products(products, output_dir, name):
+    """This currently writes the WKAPP Products to file.
+
+    """
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_baroloinput.fits'), products.baroloinput)
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_barolomod.fits'), products.barolomod)
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_barolosurfdens.fits'), products.barolosurfdens)
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_diagnosticplot.fits'), products.diagnosticplot)
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_diffcube.fits'), products.diffcube)
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_fatinput.fits'), products.fatinput)
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_fatmod.fits'), products.fatmod)
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_fullresmodcube.fits'), products.fullresmodcube)
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_fullresproccube.fits'), products.fullresproccube)
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_modcube.fits'), products.modcube)
+    _write_zipped_fits_file(os.path.join(output_dir, f'{name}_procdata.fits'), products.procdata)
+
+
 def get_slurm_output(source_name):
     """Get slurmOutput metadata for a given source
     TODO(austin): move into metadata submodule
@@ -167,18 +184,11 @@ def get_catalog(tag):
     source_field_names.remove("id")
 
     # Get sources and detections
-    sources = [
-        Source.objects.get(id=sd.source_id) for sd in [
-            SourceDetection.objects.get(id=tsd.source_detection_id) for tsd in
-            TagSourceDetection.objects.filter(tag_id=Tag.objects.get(name=tag).id)
-        ]
-    ]
-    detections = [
-        Detection.objects.get(id=sd.detection_id) for sd in [
-            SourceDetection.objects.get(id=tsd.source_detection_id) for tsd in
-            TagSourceDetection.objects.filter(tag_id=Tag.objects.get(name=tag).id)
-        ]
-    ]
+    ir_tag = Tag.objects.get(name='Internal Release')
+    sds = [tsd.source_detection for tsd in TagSourceDetection.objects.filter(tag_id=Tag.objects.get(name=tag).id)]
+    sds_released = [sd for sd in sds if ir_tag in [tsd.tag for tsd in TagSourceDetection.objects.filter(source_detection=sd)]]
+    sources = [sd.source for sd in sds_released]
+    detections = [sd.detection for sd in sds_released]
 
     # Add columns to the table
     for field in source_field_names:
@@ -226,50 +236,64 @@ def get_catalog(tag):
     return table
 
 
-def save_products_for_source(tag, source_name, *args, **kwargs):
+def save_products_for_source(id, filename, *args, **kwargs):
     """Save source finding output products for a given source name.
 
     """
-    table = get_catalog(tag)
-    try:
-        idx = list(table['name']).index(source_name)
-        row = table[idx]
-    except Exception:
-        sys.stderr.write("Could not find source with provided name in tagged data.")
-        return None
-    detection = Detection.objects.get(id=row['id'])
+    detection = Detection.objects.get(id=id)
     products = Product.objects.get(detection=detection)
-
-    name = source_name.replace(' ', '_')
-    parent = f'{name}_products'
-    if not os.path.isdir(parent):
-        os.mkdir(parent)
+    if not os.path.isdir(filename):
+        os.mkdir(filename)
 
     # Write fits files
-    _write_products(products, f'{parent}/{name}')
-
+    _write_source_products(products, filename, detection.name.replace(' ', '_'))
     return
 
 
-def save_products(tag, *args, **kwargs):
-    """Save source finding output products for a given tag
+def save_source_products(table, filename, *args, **kwargs):
+    """Save source finding output products for a given table
 
     """
-    table = get_catalog(tag)
-    parent = '%s_products' % tag.replace(' ', '_')
-    if not os.path.isdir(parent):
-        os.mkdir(parent)
+    if not os.path.isdir(filename):
+        os.mkdir(filename)
 
     for row in table:
+        id = int(row['id'])
         name = row['name'].replace(' ', '_')
-        if not os.path.isdir(f'{parent}/{name}'):
-            os.mkdir(f'{parent}/{name}')
-        detection = Detection.objects.get(id=row['id'])
-        products = Product.objects.get(detection=detection)
-        _write_products(products, f'{parent}/{name}/{name}')
+        subdir = f'{filename}/{name}'
+        if not os.path.isdir(subdir):
+            os.mkdir(subdir)
+        save_products_for_source(id, subdir)
 
-    os.system(f'tar -czf {parent}.tar.gz {parent}')
+    os.system(f'tar -czf {filename}.tar.gz {filename}')
+    return
 
+
+def save_products_for_kinematic_model(id, filename, *args, **kwargs):
+    kinematic_model = KinematicModel.objects.get(id=id)
+    products = WKAPPProduct.objects.get(kinematic_model=kinematic_model)
+    if not os.path.isdir(filename):
+        os.mkdir(filename)
+    _write_kinematic_model_products(products, filename, kinematic_model.source.name.replace(' ', '_'))
+    return
+
+
+def save_kinematic_model_products(table, filename, *args, **kwargs):
+    """Save kinematic modelling output products for a table
+
+    """
+    if not os.path.isdir(filename):
+        os.mkdir(filename)
+
+    for row in table:
+        id = int(row['id'])
+        name = row['source'].replace(' ', '_')
+        subdir = f'{filename}/{name}'
+        if not os.path.isdir(subdir):
+            os.mkdir(subdir)
+        save_products_for_kinematic_model(id, subdir)
+
+    os.system(f'tar -czf {filename}.tar.gz {filename}')
     return
 
 
